import re
from typing import Any, Dict, Optional

import woodchips

from homebrew_releaser.constants import LOGGER_NAME


class Formula:
    @staticmethod
    def generate_formula_data(
        owner: str,
        repo_name: str,
        repository: Dict[str, Any],
        checksum: str,
        install: str,
        tar_url: str,
        depends_on: Optional[str] = None,
        test: Optional[str] = None,
    ) -> str:
        """Generates the formula data for Homebrew.

        We attempt to ensure generated formula will pass `brew audit --strict --online` if given correct inputs:
        - Proper class name
        - 80 characters or less desc field (alphanumeric characters and does not start with an article)
        - Present homepage
        - URL points to the tar file
        - Checksum matches the url archive
        - Proper installable binary
        - Test is included
        - No version attribute if Homebrew can reliably infer the version from the tar URL (GitHub tag)
        """
        logger = woodchips.get(LOGGER_NAME)

        max_desc_field_length = 80  # `brew audit` wants no more than 80 characters in the desc field

        class_name = re.sub(r'[-_. ]+', '', repo_name.title())
        license_type = repository['license']['spdx_id']
        description = re.sub(r'[.!]+', '', repository['description'][:max_desc_field_length]).strip().capitalize()

        # If the first word of the desc is an article, we cut it out per `brew audit`
        articles = {
            'a',
            'an',
            'the',
        }
        first_word_of_desc = description.split(' ', 1)
        if first_word_of_desc[0].lower() in articles:
            description = first_word_of_desc[1].strip().capitalize()

        depends_on_content = ''

        if depends_on:
            dependencies = [dependency.strip() for dependency in depends_on.split('\n') if dependency]

            # `brew audit` wants dependencies sorted
            for dependency in sorted(dependencies):
                depends_on_content += f'  depends_on {dependency}\n'
            depends_on_content = f'\n{depends_on_content}'

        # RUBY TEMPLATE DATA TO REMAIN DOUBLE SPACED
        test_content = (
            f"""
  test do
    {test.strip()}
  end
end"""
            if test
            else 'end'
        )

        template = f"""# typed: false
# frozen_string_literal: true

# This file was generated by Homebrew Releaser. DO NOT EDIT.
class {class_name} < Formula
  desc "{description}"
  homepage "https://github.com/{owner}/{repo_name}"
  url "{tar_url}"
  sha256 "{checksum}"
  license "{license_type}"
{depends_on_content}
  def install
    {install.strip()}
  end
{test_content}
"""
        logger.debug('Homebrew formula generated successfully.')

        return template
